import React, { useEffect, useState } from 'react'
import {
  Box,
  Button,
  CircularProgress,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  FormControl,
  InputLabel,
  MenuItem,
  TextField
} from '@mui/material'
import * as yup from 'yup'
import { useForm } from 'react-hook-form'
import { yupResolver } from '@hookform/resolvers/yup'
import Select from '@mui/material/Select'
import useAxios from 'src/hooks/useAxios'
import { Errors } from 'src/lib/errors'
import { useRouter } from 'next/router'
import { useRecoilState } from 'recoil'
import { SuccessMessageAtom } from '@/atoms/SuccessMessageAtom'
import swal from 'sweetalert'
import CircleLoading from 'src/components/CircleLoading'
import useLanguage from 'src/hooks/useLanguage'

const LanguageForm = ({ onClose, onRevalidate, languageId = 0 }) => {
  const [open, setOpen] = useState(true)
  const [loading, setLoading] = useState(false)
  const [modalLoading, setModalLoading] = useState(false)
  const message = useRecoilState(SuccessMessageAtom)
  const [language, setLanguage] = useState(null)
  const router = useRouter()
  const request = useAxios()
  const { translate } = useLanguage()

  const handleClose = () => {
    setOpen(false)
    onClose(false)
  }

  const schema = yup.object().shape({
    name: yup.string().required().min(3),
    abbr: yup.string().required(),
    direction: yup.string().required().min(3),
    status: yup.boolean().nullable()
  })

  const {
    register,
    setValue,
    setError,
    formState: { errors },
    handleSubmit,
    reset
  } = useForm({
    resolver: yupResolver(schema)
  })

  const onSubmit = data => {
    setLoading(true)
    request({
      url: languageId === 0 ? 'language' : `language/${languageId}`,
      method: languageId === 0 ? 'POST' : 'PUT',
      data: data
    })
      .then(res => {
        setLoading(false)
        if (res.data.result) {
          message[1](res.data.message)
          reset()
          onRevalidate(true)
          handleClose()
        } else {
          swal({
            icon: 'error',
            title: 'Something went wrong',
            text: res.data.message,
            autoHide: true
          })
        }
      })
      .catch(err => {
        Errors(err, setError, router)
        setLoading(false)
      })
  }

  useEffect(() => {
    if (languageId !== 0) {
      setModalLoading(true)
      axios
        .get(`language/${languageId}`)
        .then(res => {
          if (res.status === 200) {
            setLanguage(res.data)
            setValue('name', res.data.name)
            setValue('abbr', res.data.abbr)
            setValue('direction', res.data.direction)
            setValue('status', res.data.status)
          }
          setModalLoading(false)
        })
        .catch(err => {
          Errors(err, setError, router)
          setModalLoading(false)
        })
    }
  }, [languageId])

  return (
    <Dialog open={open} keepMounted onClose={handleClose} aria-describedby='alert-dialog-slide-description'>
      <Box component={'form'} onSubmit={handleSubmit(onSubmit)} visibility autoComplete='off'>
        <DialogTitle>{languageId === 0 ? translate('Create new language') : translate('Edit language')}</DialogTitle>
        <DialogContent>
          <DialogContentText id='alert-dialog-slide-description'>
            {modalLoading ? (
              <div className={'py-8 px-16'}>
                <CircleLoading />
              </div>
            ) : (
              <>
                <div className={'mt-3'}>
                  <TextField
                    {...register('name')}
                    size={'small'}
                    id='outlined-basic'
                    label={translate('Language name')}
                    variant='outlined'
                  />
                  {errors['name'] && (
                    <p className={'ml-1 p-1 text-xs text-red-500'}>{errors['name']?.message.replace('_', ' ')}</p>
                  )}
                </div>
                <div className={'mt-3'}>
                  <TextField
                    {...register('abbr')}
                    size={'small'}
                    id='outlined-basic2'
                    label={translate('Abbreviation')}
                    variant='outlined'
                  />
                  {errors['abbr'] && (
                    <p className={'ml-1 p-1 text-xs text-red-500'}>{errors['abbr']?.message.replace('_', ' ')}</p>
                  )}
                </div>
                <div className={'mt-3'}>
                  <FormControl size={'small'} fullWidth>
                    <InputLabel id='demo-simple-select-label'>{translate('Direction')}</InputLabel>
                    <Select
                      defaultValue={language?.direction}
                      size={'small'}
                      labelId='demo-simple-select-label'
                      id='demo-simple-select'
                      label='Age'
                      onChange={e => {
                        setValue('direction', e.target.value)
                      }}
                    >
                      <MenuItem value={''}>{translate('Direction')}</MenuItem>
                      <MenuItem value={'rtl'}>{translate('Right to left')}</MenuItem>
                      <MenuItem value={'ltr'}>{translate('Left to right')}</MenuItem>
                    </Select>
                  </FormControl>
                  {errors['direction'] && (
                    <p className={'ml-1 p-1 text-xs text-red-500'}>{errors['direction']?.message.replace('_', ' ')}</p>
                  )}
                </div>
                {languageId !== 0 && (
                  <div className={'mt-3'}>
                    <FormControl size={'small'} fullWidth>
                      <InputLabel id='demo-simple-select-label'>Status</InputLabel>
                      <Select
                        defaultValue={language?.status}
                        size={'small'}
                        labelId='demo-simple-select-label'
                        id='demo-simple-select'
                        label='Age'
                        onChange={e => {
                          setValue('status', e.target.value)
                        }}
                      >
                        <MenuItem value={true}>Active</MenuItem>
                        <MenuItem value={false}>Inactive</MenuItem>
                      </Select>
                    </FormControl>
                    {errors['direction'] && (
                      <p className={'ml-1 p-1 text-xs text-red-500'}>
                        {errors['direction']?.message.replace('_', ' ')}
                      </p>
                    )}
                  </div>
                )}
              </>
            )}
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => handleClose()} color={'error'} variant={'outlined'} size={'small'}>
            Close
          </Button>
          <Button type={'submit'} disabled={loading} color={'primary'} variant={'outlined'} size={'small'}>
            {loading ? <CircularProgress size={23} /> : <span>Save</span>}
          </Button>
        </DialogActions>
      </Box>
    </Dialog>
  )
}

export default LanguageForm
